@@ -26,10 +26,14 @@
 #include <gst/interfaces/colorbalance.h>
 #include <gst/interfaces/xoverlay.h>
 #include <gst/interfaces/videoorientation.h>
+#include <gst/video/video.h>
 
 #include <Lum/Base/Util.h>
 
+#include <Lum/Dialog.h>
+
 #include <Lum/OS/Display.h>
+#include <Lum/OS/EventLoopGMainLoop.h>
 
 #include <Lum/OS/X11/Display.h>
 
@@ -45,8 +49,17 @@ static gboolean cb_have_data(GstElement* /*element*/,
   return true;
 }
 
+gboolean GstCallback(GstBus* bus, GstMessage* message, gpointer user_data)
+{
+  static_cast<VideoControl*>(user_data)->OnGstMessage(message);
+
+  return true;
+}
+
 VideoControl::VideoControl()
 : pipeline(NULL),videoSink(NULL),imageSink(NULL),
+  source(NULL),
+  videoWidth(0),videoHeight(0),
   storeImage(false),done(false),
   gstMsg(new Lum::Model::Action()),
   knipsed(new Lum::Model::Action()),
@@ -60,25 +73,27 @@ VideoControl::VideoControl()
 
 VideoControl::~VideoControl()
 {
-  UnattachModel(timer);
-  UnattachModel(gstMsg);
+  if (pipeline!=NULL) {
+    gst_element_set_state(pipeline,GST_STATE_NULL);
 
-  dynamic_cast<Lum::OS::X11::Display*>(Lum::OS::display)->RemoveGstListener(gstMsg.Get());
+    while (true) {
+      GstState current;
 
-  ::gst_element_set_state(pipeline,GST_STATE_NULL);
-
-  while (true) {
-    ::GstState current;
-
-    ::gst_element_get_state(pipeline,&current,NULL,GST_CLOCK_TIME_NONE);
-    if (current==GST_STATE_NULL) {
-      break;
+      gst_element_get_state(pipeline,&current,NULL,GST_CLOCK_TIME_NONE);
+      if (current==GST_STATE_NULL) {
+        break;
+      }
+      Lum::Base::SystemTime::Sleep(1);
     }
-    Lum::Base::SystemTime::Sleep(1);
+
+    gst_object_unref(pipeline);
+    pipeline=NULL;
   }
 
-  ::gst_object_unref(pipeline);
-  pipeline=NULL;
+  if (source!=NULL) {
+    g_source_destroy(source);
+    g_source_unref(source);
+  }
 }
 
 void VideoControl::CheckCameraOrientation()
@@ -108,7 +123,7 @@ void VideoControl::GotImage(unsigned char* data)
 {
   Lum::OS::MutexGuard guard(mutex);
 
-  if (!storeImage) {
+  if (!storeImage || videoWidth==0 || videoHeight==0) {
     return;
   }
 
@@ -116,16 +131,16 @@ void VideoControl::GotImage(unsigned char* data)
 
   image=Lum::Images::Factory::factory->CreateImage();
 
-  pixel=new Lum::Images::Pixel[640*480];
+  pixel=new Lum::Images::Pixel[videoWidth*videoHeight];
 
-  for (size_t i=0; i<640*480; i++) {
+  for (size_t i=0; i<videoWidth*videoHeight; i++) {
     pixel[i].r=data[i*3];
     pixel[i].g=data[i*3+1];
     pixel[i].b=data[i*3+2];
     pixel[i].a=255;
   }
 
-  image->SetData(640,480,false,pixel);
+  image->SetData(videoWidth,videoHeight,false,pixel);
 
   storeImage=false;
 
@@ -154,6 +169,8 @@ void VideoControl::CalcSize()
   width=320;
   height=240;
 
+  AttachModel(dynamic_cast<Lum::Dialog*>(GetWindow()->GetMaster())->GetOpenedAction());
+
   minWidth=width;
   minHeight=height;
 
@@ -163,20 +180,19 @@ void VideoControl::CalcSize()
 void VideoControl::Layout()
 {
   if (!done) {
-    ::GError *error=NULL;
+    GError *error=NULL;
 
     gst_init(NULL,NULL);
 
     // Pipeline elements
     /*
-    pipeline=::gst_parse_launch("v4lsrc name=videosource ! "
+    pipeline=::gst_parse_launch("videotestsrc name=videosource ! "
                                 "video/x-raw-yuv ! "
                                 "ffmpegcolorspace ! "
                                 "tee name=t ! "
                                 "queue ! "
                                 "{ xvimagesink name=videosink }"
-                                "t. !"
-                                 "queue ! "
+                                "t. ! queue ! "
                                 "{ ffmpegcolorspace ! "
                                 "video/x-raw-rgb,bpp=24,depth=24,framerate:fraction=8/1 !"
                                 "fakesink name=imagesink }",
@@ -190,7 +206,7 @@ void VideoControl::Layout()
                                 "t. ! "
                                 "queue ! "
                                 "{ ffmpegcolorspace ! "
-                                "video/x-raw-rgb,width=640,height=480,bpp=24,depth=24,framerate=(fraction)8/1 ! "
+                                "video/x-raw-rgb,bpp=24,depth=24,framerate=(fraction)8/1 ! "
                                 "fakesink name=imagesink }",
                                 &error);
 
@@ -208,11 +224,32 @@ void VideoControl::Layout()
     g_object_set(G_OBJECT(imageSink),"signal-handoffs",TRUE,NULL);
     g_signal_connect(imageSink,"handoff",G_CALLBACK(cb_have_data),this);
 
-    dynamic_cast<Lum::OS::X11::Display*>(Lum::OS::display)->AddGstListener(pipeline,gstMsg.Get());
-    dynamic_cast<Lum::OS::X11::Display*>(Lum::OS::display)->AddGstListener(videoSink,gstMsg.Get(),false);
+    Lum::OS::GMainLoopEventLoop* eventLoop=dynamic_cast<Lum::OS::GMainLoopEventLoop*>(Lum::OS::display->GetEventLoop());
+    GstBus*                      bus=gst_element_get_bus(pipeline);
+
+    source=gst_bus_create_watch(bus);
+    g_source_set_callback(source,(GSourceFunc)GstCallback,this,NULL);
+    g_source_attach(source,eventLoop->context);
+    gst_object_unref(GST_OBJECT(bus));
 
     gst_element_set_state(pipeline,GST_STATE_PLAYING);
 
+    GstPad *videoSourcePad;
+
+    videoSourcePad=gst_element_get_static_pad(videoSource,"src");
+
+    if (videoSourcePad!=NULL) {
+      gint w,h;
+
+      if (gst_video_get_size(videoSourcePad,&w,&h)) {
+        videoWidth=w;
+        videoHeight=h;
+      }
+
+      gst_object_unref(videoSourcePad);
+    }
+
+
     Lum::OS::display->AddTimer(1,0,timer);
 
     done=true;
@@ -237,60 +274,59 @@ void VideoControl::CancelRequestImage()
   image=NULL;
 }
 
+void VideoControl::OnGstMessage(GstMessage* message)
+{
+  //std::cout << "Message: " << gst_message_type_get_name(GST_MESSAGE_TYPE(message)) << std::endl;
+
+  if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_ERROR) {
+    GError *error;
+    char   *text;
+
+    gst_message_parse_error(message,&error,&text);
+
+    std::cerr << "gstreamer ERROR: " << text << std::endl;
+
+    g_free(error);
+    g_free(text);
+  }
+  else if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_WARNING) {
+    GError *error;
+    char   *text;
+
+    gst_message_parse_warning(message,&error,&text);
+
+    std::cerr << "gstreamer WARNING: " << text << std::endl;
+
+    g_free(error);
+    g_free(text);
+  }
+  else if (message->src==GST_OBJECT(pipeline) && GST_MESSAGE_TYPE(message)==GST_MESSAGE_STATE_CHANGED) {
+    GstState newState;
+
+    gst_message_parse_state_changed(message,NULL,&newState,NULL);
+
+    if (newState==GST_STATE_PLAYING || newState==GST_STATE_PAUSED) {
+      knips->Enable();
+    }
+    else {
+      knips->Disable();
+    }
+  }
+}
+
 void VideoControl::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
 {
-  if (model==gstMsg && gstMsg->IsFinished()) {
-    GstMessage *message=static_cast<const Lum::OS::X11::Display::GstMsg*>(&msg)->message;
-
-    if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_ERROR) {
-      GError *error;
-      char   *text;
-
-      gst_message_parse_error(message,&error,&text);
-
-      std::cerr << "gstreamer ERROR: " << text << std::endl;
-
-      g_free(error);
-      g_free(text);
-    }
-		else if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_WARNING) {
-      GError *error;
-      char   *text;
-
-      gst_message_parse_warning(message,&error,&text);
-
-      std::cerr << "gstreamer WARNING: " << text << std::endl;
-
-      g_free(error);
-      g_free(text);
-    }
-   else if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_ELEMENT) {
-      if (gst_message_get_structure(message)!=NULL) {
-        if (strcmp("prepare-xwindow-id",gst_structure_get_name(gst_message_get_structure(message)))==0) {
-          gst_x_overlay_set_xwindow_id(GST_X_OVERLAY(videoSink),window);
-        }
-      }
-    }
-    else if (message->src==GST_OBJECT(pipeline) && GST_MESSAGE_TYPE(message)==GST_MESSAGE_STATE_CHANGED) {
-      GstState newState;
-
-      gst_message_parse_state_changed(message,NULL,&newState,NULL);
-
-      if (newState==GST_STATE_PLAYING || newState==GST_STATE_PAUSED) {
-        knips->Enable();
-      }
-      else {
-        knips->Disable();
-      }
-    }
-
-    // Since we might be called from another thread, make sure that we don't call the baseclass
-    return;
-  }
-  else if (model==timer && timer->IsFinished()) {
+  if (model==timer && timer->IsFinished()) {
     CheckCameraOrientation();
     Lum::OS::display->AddTimer(1,0,timer);
   }
+  else if (GetWindow()!=NULL &&
+           dynamic_cast<Lum::Dialog*>(GetWindow()->GetMaster())!=NULL &&
+           model==dynamic_cast<Lum::Dialog*>(GetWindow()->GetMaster())->GetOpenedAction() &&
+           dynamic_cast<Lum::Dialog*>(GetWindow()->GetMaster())->GetOpenedAction()->IsFinished() &&
+           videoSink!=NULL) {
+    gst_x_overlay_set_xwindow_id(GST_X_OVERLAY(videoSink),window);
+  }
 
   Object::Resync(model,msg);
 }
