@@ -28,7 +28,6 @@
 #include <Lum/Space.h>
 #include <Lum/String.h>
 #include <Lum/Text.h>
-#include <Lum/WindowGroup.h>
 
 #include <Lum/Base/DateTime.h>
 #include <Lum/Base/Path.h>
@@ -75,9 +74,6 @@ public:
     Lum::DirNameSelect *dirSel;
     Lum::Label         *label;
     Lum::Panel         *panel;
-    Lum::WindowGroup   *wGroup;
-
-    wGroup=new Lum::WindowGroup();
 
     panel=new Lum::VPanel();
 
@@ -107,9 +103,7 @@ public:
 
     panel->Add(bRow);
 
-    wGroup->SetMain(panel);
-
-    SetTop(wGroup);
+    SetMain(panel);
 
     Dialog::PreInit();
   }
@@ -151,17 +145,6 @@ public:
     AttachModel(about);
   }
 
-  ~MainWindow()
-  {
-    UnattachModel(about);
-    UnattachModel(settings);
-    UnattachModel(control->GetKnipsedAction());
-    UnattachModel(knips);
-    UnattachModel(GetOpenedAction());
-
-    UnattachModel(knipsTimer);
-  }
-
   void StoreImage()
   {
     assert(imageDirectory.Valid() && imagePrefix.Valid());
@@ -253,10 +236,6 @@ public:
     Lum::Panel         *vpanel,*vpanel2,*hpanel;
     Lum::String        *string;
     Lum::Text          *text;
-    Lum::WindowGroup   *wGroup;
-
-    wGroup=new Lum::WindowGroup();
-    wGroup->SetFlex(true,true);
 
     vpanel=new Lum::VPanel();
     vpanel->SetFlex(true,true);
@@ -327,8 +306,7 @@ public:
 
     vpanel->Add(hpanel);
 
-    wGroup->SetMain(vpanel);
-    SetTop(wGroup);
+    SetMain(vpanel);
 
     Dialog::PreInit();
   }
